import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

public class RampProgram implements ActionListener
{
	private JTextField txtRange;
	private JTextField txtAcceleration;
	private JTextField txtDecceleration;
	private JTextField txtMinDelay;
	private JTextField txtMaxDelay;
	private JButton btnRedraw;
	private RampChart chart;

	private class RampSession
	{
		public int currentStep;
		public int totalSteps;
		public int minimumDelay;
		public int maximumDelay;
		public int acceleration;
		public int decceleration;

		public int currentDelay;
		public boolean intersect;
		public int pointA, pointB;
		public boolean initilized;

		public RampSession (int tot, int acc, int dec, int min,
			int max) throws Exception
		{
			if (min > max) {
				throw new Exception("Maximum delay must exceed or be equal to minimum delay.");
			}

			currentStep = 0;
			totalSteps = tot;
			minimumDelay = min;
			maximumDelay = max;
			acceleration = acc;
			decceleration = dec;
			currentDelay = 0;
			intersect = false;
			pointA = 0;
			pointB = 0;

			init();
		}

		private void init ()
		{
			int deltaDelay = maximumDelay - minimumDelay;
			int rampTime = (deltaDelay / -acceleration +
				deltaDelay / decceleration);

			if (totalSteps < rampTime) {
				// We should calculate the intersection point.
				intersect = true;

				pointA = pointB = -((decceleration * totalSteps) /
					(acceleration - decceleration));

			} else {
				// We have enough time to do a proper ramping.
				pointA = deltaDelay / -acceleration;
				pointB = totalSteps - (deltaDelay / decceleration);
			}

			currentDelay = maximumDelay;
		}

		public int getDelay ()
		{
			if (currentStep < pointA) {
				currentDelay += acceleration;

			} else if (currentStep > pointB) {
				currentDelay += decceleration;

			}
			
			++currentStep;

			return currentDelay;
		}


	}

	public RampProgram ()
	{
		txtRange = new JTextField("8", 3);
		txtAcceleration = new JTextField("1", 3);
		txtDecceleration = new JTextField("1", 3);
		txtMinDelay = new JTextField("0", 3);
		txtMaxDelay = new JTextField("5", 3);
		btnRedraw = new JButton("Redraw");

		gui();
	}

	private void gui()
	{
		JFrame.setDefaultLookAndFeelDecorated(true);

		JFrame frame = new JFrame("RampProgram");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		JPanel rootPanel = new JPanel(new BorderLayout());
		
		chart = new RampChart();
		chart.setOpaque(true);
		chart.setBackground(Color.WHITE);
		
		rootPanel.add(chart, BorderLayout.CENTER);

		JPanel buttonPanel = new JPanel();
		buttonPanel.add(new JLabel("Range: "));
		buttonPanel.add(txtRange);
		buttonPanel.add(new JLabel("Acceleration: "));
		buttonPanel.add(txtAcceleration);
		buttonPanel.add(new JLabel("Decceleration: "));
		buttonPanel.add(txtDecceleration);
		buttonPanel.add(new JLabel("Maxdelay: "));
		buttonPanel.add(txtMaxDelay);
		buttonPanel.add(new JLabel("Mindelay: "));
		buttonPanel.add(txtMinDelay);
		buttonPanel.add(btnRedraw);

		btnRedraw.addActionListener(this);

		rootPanel.add(buttonPanel, BorderLayout.PAGE_END);

		frame.setContentPane(rootPanel);

		frame.pack();
		frame.setVisible(true);
	}

	public void actionPerformed (ActionEvent e)
	{
		Vector<Integer> newValues = new Vector<Integer>();

		int range = Integer.parseInt(txtRange.getText());
		int acc = Integer.parseInt(txtAcceleration.getText());
		int dec = Integer.parseInt(txtDecceleration.getText());
		int min = Integer.parseInt(txtMinDelay.getText());
		int max = Integer.parseInt(txtMaxDelay.getText());
		
		RampSession session = null;

		try {
			session = new RampSession(range, -acc, dec, min, max);
		} catch (Exception ex) {
			System.err.println("Maximum value is lower than minimum value.");
		}

		for (int i = 0; i < range; ++i) {
			newValues.add(session.getDelay());
		}

		chart.setValues(newValues);
		
	}

	public static void main (String[] args)
	{
		new RampProgram();
	}

}

